@@ -18,10 +18,15 @@ public:
 
 private:
     Ui::CPS2emu *ui;
-    QString path;    
+    QString path;
+    //QString recentPath;
     QProcess *cps2cache;
+    QProcess *cps2run;
     bool romcnv;
+    bool running;
     bool checkCache();
+    bool checkNVRAM();    
+    bool checkHiScore();
     short int i;
 
 private slots:
