@@ -419,8 +419,11 @@ static int load_rom_info(const char *game_name)
 	machine_input_type   = 0;
 	machine_init_type    = 0;
 	machine_screen_type  = 0;
-
+#ifdef N900
+	sprintf(path, "%srominfo.cps2", "/opt/cps2emu/config/");
+#else
 	sprintf(path, "%srominfo.cps2", launchDir);
+#endif
 
 	if ((fp = fopen(path, "r")) != NULL)
 	{
