@@ -1,3 +1,7 @@
+#include <stdio.h>
+#include <string.h>
+#include "c68k.h"
+
 	typedef struct
 	{
 		void *label; /* opcode label */
