@@ -16,8 +16,11 @@
 #else
 #include <SDL/SDL.h>
 #endif
+
 #ifdef N900
 #include "maemo.h"
+// I know howto use HAA to scale SDL?
+// #include <SDL/SDL_haa.h>
 #endif
 
 #define EMULATOR_TITLE      "CPS2EMU for GNU/Linux Maemo 5(N900)"
@@ -82,6 +85,7 @@ u16 *work_frame = NULL;
 //Sixaxis support
 #ifdef N900
 int sixaxis = 0;
+int state_load_atstart = 0;
 #endif
 
 /******************************************************************************
@@ -394,6 +398,15 @@ int main(int argc, char *argv[])
 	option_cpuspeed = 0;
 	option_hiscore = 1;
 
+#ifdef N900
+    //PS3 Sixaxis support, for N900
+    sixaxis = file_exists("/dev/input/js0");
+    if (sixaxis)
+        msg_printf("PS3 controller found! All keys remapped!\n");
+    else
+        msg_printf("Using N900 keyboard!\n");
+#endif
+
 #ifdef GP2X
 	if(realpath("/proc/self/exe", launchDir) != NULL) {
 		char *p = strrchr(launchDir, '/');
@@ -530,7 +543,7 @@ int main(int argc, char *argv[])
 		exit (1);
 	}
 
-	screen_surface = SDL_SetVideoMode (320, 240, 16, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN);
+        screen_surface = SDL_SetVideoMode (320, 240, 16, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN);
 	if (screen_surface == NULL) {
 		fprintf (stderr, "Couldn't set video mode: %s\n", SDL_GetError ());
 		exit (2);
@@ -562,7 +575,7 @@ int main(int argc, char *argv[])
 	state_slot = 0;
 	service_mode = 0;
 
-	cps2_main();
+        cps2_main();
 }
 
 
@@ -770,7 +783,7 @@ u32 reset_joystick(int num, int max_buttons, int rotate)
 u32 read_joystick(int num)
 {
 	u32 buttons = 0;
-	int i;
+        char paused =-1;
 
 #ifdef GP2X
 	if (num == 0) {
@@ -876,23 +889,26 @@ u32 read_joystick(int num)
 	if(num != 0) return 0;    
 
 	SDL_Delay(1);
+        paused = -1;
 	while(SDL_PollEvent(&event)) {
 		switch(event.type) {
 			case SDL_QUIT:
 				Loop = LOOP_EXIT;
+                                state_save(10);
 				break;
-			case SDL_KEYDOWN:
+                case SDL_KEYDOWN:
 				//printf("KEY = %c , %d\n", event.key.keysym.sym, event.key.keysym.sym);
 				switch(event.key.keysym.sym) {
-					case SDLK_BACKSPACE:                    
-                        Loop = LOOP_EXIT;
-                        break;
-					case 'm':
-                        Loop = LOOP_RESET;
-                        break;
-					case SDLK_F7:
-                        menu_mode = 1;
-                        break;
+                                case SDLK_BACKSPACE:
+                                    Loop = LOOP_EXIT;
+                                    state_save(10);
+                                    break;
+                                case 'm':
+                                    Loop = LOOP_RESET;
+                                    break;
+                                case 'l':
+                                    menu_mode = 1;
+                                    break;
 				}
 			case SDL_KEYUP:
 				button = -1;
@@ -939,6 +955,7 @@ u32 read_joystick(int num)
                         //Exit emulator on press (PS) key ;-)
                         case 'd': //(PS) key!                        
                             Loop = LOOP_EXIT;
+                            state_save(10);
                             break;
                     }
                 }
@@ -963,7 +980,6 @@ u32 read_joystick(int num)
                         case SDLK_RIGHT: button = P1_RIGHT; break;
                         case SDLK_UP: button = P1_UP; break;
                         case SDLK_DOWN: button = P1_DOWN; break;
-                        //case SDLK_F2:
                         case 'o':
                             button = SERV_SWITCH;
                             msg_printf("Enter service mode\n");
@@ -994,9 +1010,7 @@ u32 read_joystick(int num)
 }
 
 
-static TICKER get_ticks(void)
-{
-#define N900 1
+static TICKER get_ticks(void){
 #ifdef N900 
 	struct timeval tv;
 	gettimeofday(&tv, 0);
@@ -1230,12 +1244,19 @@ void update_screen(void)
 		if(state_slot < 0x10) {
 			state_save(state_slot & 0x0f);
 		} else {
-			state_load(state_slot & 0x0f);
+                        state_load(state_slot & 0x0f);
 		}
 		autoframeskip_reset();
 		state_slot = -1;
 	}
 */
+        //Autoload state 10 at statup
+        if(state_load_atstart == 0){
+            state_load_atstart = 1;
+            state_load(10);
+            autoframeskip_reset();
+        }
+
 }
 
 
@@ -1273,9 +1294,32 @@ void SetTextMode(void)
 	gp2xregs[0x290A>>1] = vsc >> 16;
 	gp2xregs[0x290C>>1] = hw;
 #else
+        //HAA_Actor *actor;
+
 	if(screen_surface->w != 320) {
-		SDL_FreeSurface(screen_surface);
-		screen_surface = SDL_SetVideoMode (320, 240, 16, SDL_HWSURFACE | SDL_DOUBLEBUF);
+            //option_rescale = 2;
+            SDL_FreeSurface(screen_surface);
+            //HAA_Init(SDL_FULLSCREEN);
+            //screen_surface = SDL_SetVideoMode (640, 480, 16, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN);
+            screen_surface = SDL_SetVideoMode (320, 240, 16, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN);
+            //screen_surface = SDL_SetVideoMode (0, 0, 16, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN);
+            //screen_surface = SDL_SetVideoMode (768, 448, 16, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN);
+
+            /*
+             * I not have success on use HAA! :-(
+            //actor = HAA_CreateActor(0, 640, 480, 16);
+            actor = HAA_CreateActor(SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN, 320, 240, 16);
+
+            HAA_SetPosition(actor, 0, 0);
+            HAA_SetScale(actor, 1.0, 1.0);
+            HAA_Show(actor);
+            HAA_SetVideoMode();
+
+            //screen_surface = SDL_SetVideoMode (640, 480, 16, SDL_SWSURFACE);
+            //screen_surface = SDL_SetVideoMode (320, 240, 16, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN);
+            //screen = actor->surface;
+            screen_surface = actor->surface;
+            */
 	}
 #endif
 }
@@ -1328,7 +1372,7 @@ void SetGameMode(void)
 	if((option_rescale >= 2) && (screen_surface->w != 384)) {
 		SDL_FreeSurface(screen_surface);
 		//screen_surface = SDL_SetVideoMode (384, option_rescale == 2 ? 224 : 240, 16, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN);
-		screen_surface = SDL_SetVideoMode (768, 448, 16, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN);
+                screen_surface = SDL_SetVideoMode (768, 448, 16, SDL_HWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN);
 	}
 #endif
 }
@@ -1339,29 +1383,19 @@ void SetGameMode(void)
 
 void msg_screen_init()
 {
-	SetTextMode();
+        SetTextMode();
 #ifndef GP2X
 	SDL_LockSurface(screen_surface);
 	screen = screen_surface->pixels;
 #endif
-	memset(screen, 0x12, 320 * 8 * 2);
-	memset(screen + 320 * 8, 0, 320 * 232 * 2);
+        memset(screen, 0x12, 320 * 8 * 2);
+        memset(screen + 320 * 8, 0, 320 * 232 * 2);
 #ifndef GP2X
 	SDL_UnlockSurface(screen_surface);
 	screen = NULL;
 #endif
 	msg_line = 0;
-	msg_printf(EMULATOR_TITLE);
-    
-#ifdef N900
-    //PS3 Sixaxis support, for N900
-    sixaxis = file_exists("/dev/input/js0");
-    if (sixaxis)
-        msg_printf("PS3 controller found! All keys remapped!\n");
-    else
-        msg_printf("Using N900 keyboard!\n");
-#endif
-    
+        msg_printf(EMULATOR_TITLE);
 }
 
 void msg_screen_clear()
@@ -1413,19 +1447,19 @@ static const char *draw_text(const char *text, int x, int y, int fg, int bg)
 	screen = screen_surface->pixels;
 #endif
 
-	dst = screen + y * 320;
+        dst = screen + y * 320;
 
 	while(*p) {
 		if((*p == 0x20) && (trans)) {
-			x += 8;
+                        x += 8;
 		} else if(*p >= 0x20) {
 			int w, h;
-			u8 *font = (u8 *)&font_data[(*p - 0x20) * 8];
+                        u8 *font = (u8 *)&font_data[(*p - 0x20) * 8];
 			u16 *put = dst + x;
 
-			for(h = 8; h; --h) {
+                        for(h = 8; h; --h) {
 				u8 c = *font++;
-				for(w = 8; w; --w, ++put) {
+                                for(w = 8; w; --w, ++put) {
 					if(c & 0x80) {
 						*put = fg;
 					} else if(!trans) {
@@ -1433,12 +1467,12 @@ static const char *draw_text(const char *text, int x, int y, int fg, int bg)
 					}
 					c <<= 1;
 				}
-				put += 320 - 8;
+                                put += 320 - 8;
 			}
-			x += 8;
+                        x += 8;
 		}
 		++p;
-		if(x > 312)
+                if(x > 312)
 			break;
 	}
 
@@ -1467,7 +1501,7 @@ static void clean_msg()
 		SDL_FillRect(screen_surface, &r, title_bg);
 		title_show = 0;
 	}
-	SDL_Rect r = {0, 232, 320, 8};
+        SDL_Rect r = {0, 232, 320, 8};
 	SDL_FillRect(screen_surface, &r, 0x0000);
 	SDL_Flip(screen_surface);
 #endif
@@ -1504,7 +1538,7 @@ void msg_printf(const char *text, ...)
 	do {
 		if(msg_line != -1) {
 			++msg_line;
-			if(y == 240) {
+                        if(y == 240) {
 #ifndef GP2X
 				SDL_LockSurface(screen_surface);
 				screen = screen_surface->pixels;
@@ -1521,7 +1555,7 @@ void msg_printf(const char *text, ...)
 #ifdef GP2X
 				memset(screen + y * 320, 0, 320 * 8 * 2);
 #else
-				SDL_Rect r = {0, y, 320, 8};
+                                SDL_Rect r = {0, y, 320, 8};
 				SDL_FillRect(screen_surface, &r, 0);
 #endif
 			}
@@ -1529,7 +1563,7 @@ void msg_printf(const char *text, ...)
 #ifdef GP2X
 			memset(screen + y * 320, 0x00, 320 * 8 * 2);
 #else
-			SDL_Rect r = {0, y, 320, 8};
+                        SDL_Rect r = {0, y, 320, 8};
 			SDL_FillRect(screen_surface, &r, 0x0000);
 #endif
 		}
@@ -1575,6 +1609,7 @@ void button_wait()
 			switch(event.type) {
 				case SDL_QUIT:
 					Loop = LOOP_EXIT;
+                                        state_save(10);
 					return;
 				case SDL_KEYDOWN:
 				case SDL_KEYUP:
@@ -1602,8 +1637,8 @@ void menu_init()
 	SDL_LockSurface(screen_surface);
 	screen = screen_surface->pixels;
 #endif
-	memset(screen, 0x12, 320 * 8 * 2);
-	memset(screen + 320 * 8, 0, 320 * 232 * 2);
+        memset(screen, 0x12, 320 * 8 * 2);
+        memset(screen + 320 * 8, 0, 320 * 232 * 2);
 #ifndef GP2X
 	SDL_UnlockSurface(screen_surface);
 	screen = NULL;
@@ -1677,7 +1712,7 @@ void menu_display(void)
 	memset(buffer, 0, sizeof(buffer));
 	screen = buffer;
 #else
-	SDL_Rect rect = {0, 8, 320, 224};
+        SDL_Rect rect = {0, 8, 320, 224};
 	SDL_FillRect(screen_surface, &rect, 0);
 #endif
 
@@ -1763,32 +1798,51 @@ int menu_button(void)
 
 	SDL_Delay(1);
 	while(SDL_PollEvent(&event)) {
-		switch(event.type) {
-			case SDL_QUIT:
-				Loop = LOOP_EXIT;
-				break;
-			case SDL_KEYDOWN:
-				switch(event.key.keysym.sym) {
-					case SDLK_ESCAPE: Loop = LOOP_EXIT; break;
-				}
-			case SDL_KEYUP:
-				button = -1;
-				switch(event.key.keysym.sym) {
-					case SDLK_UP: button = MENU_UP; break;
-					case SDLK_DOWN: button = MENU_DOWN; break;
-					case SDLK_LEFT: button = MENU_LEFT; break;
-					case SDLK_RIGHT: button = MENU_RIGHT; break;
-					case SDLK_RETURN: button = MENU_SELECT; break;
-					case 'a': button = MENU_SELECT; break;
-					case SDLK_F12: button = MENU_TOGAME; break;
-				}
-				if(button != -1)
-					if(event.type == SDL_KEYDOWN)
-						menu_pressed |= BTN(button);
-					else
-						menu_pressed &= ~BTN(button);
-				break;
-		}
+            switch(event.type) {
+            case SDL_QUIT:
+                Loop = LOOP_EXIT;
+                state_save(10);
+                break;
+            case SDL_KEYDOWN:
+                button = -1;
+                if(sixaxis){ //PS3 sixaxis support
+                    switch(event.key.keysym.sym) {
+                    case 'd': //(PS) key!
+                        button = MENU_TOGAME;
+                        break;
+                    }
+                }
+            case SDL_KEYUP:
+                switch(event.key.keysym.sym) {
+                case SDLK_UP:
+                    button = MENU_UP;
+                    break;
+                case SDLK_DOWN:
+                    button = MENU_DOWN;
+                    break;
+                case SDLK_LEFT:
+                    button = MENU_LEFT;
+                    break;
+                case SDLK_RIGHT:
+                    button = MENU_RIGHT;
+                    break;
+                case SDLK_RETURN:
+                    button = MENU_SELECT;
+                    break;
+                case 'a':
+                    button = MENU_SELECT;
+                    break;
+                case SDLK_BACKSPACE:
+                    button = MENU_TOGAME;
+                    break;
+                }
+                if(button != -1)
+                    if(event.type == SDL_KEYDOWN)
+                        menu_pressed |= BTN(button);
+                else
+                    menu_pressed &= ~BTN(button);
+                break;
+            }
 	}
 #endif
 	return menu_pressed;
@@ -1829,7 +1883,7 @@ void menu_on(void)
 			if(state_slot > 9) state_slot = 0;
 			if(menu_frameskip < -1) menu_frameskip = -1;
 			if(menu_frameskip > 11) menu_frameskip = 11;
-			option_rescale &= 0x03;
+                        option_rescale &= 0x03;
 			menu_display();
 			menu_draw = 0;
 		}
@@ -1892,7 +1946,10 @@ void menu_on(void)
 				case MENU_USBREFRESH: open_joystick(); break;
 				case MENU_RESET: Loop = LOOP_RESET; menu_do = 0; break;
 				case MENU_RETURN: menu_do = 0; break;
-				case MENU_EXIT: Loop = LOOP_EXIT; break;
+                                case MENU_EXIT:
+                                    Loop = LOOP_EXIT;
+                                    state_save(10);
+                                    break;
 			}
 		}
 
